/* start of file CharPoly.c --------------------------------------------- */

/* <summary>
	SUBROUTINE CharPoly - Computes the Characteristic polynomial p of an
	n by n complex matrix a
   </summary>
*/

#include "CharPoly.h"

float _Complex SetTo(float real, float imag);

void CharPoly(float _Complex a[][maxA], int n, float _Complex p[])
/* p = Characteristic polynomial of an n by n complex matrix a, n > 1
*/
{
	int i, j, k, m;
	float _Complex s;
	float _Complex t0;
	float _Complex b[maxA][maxA];
	float _Complex c[maxA][maxA];
	float _Complex cZero = SetTo(0.0F, 0.0F);
	float _Complex cOne = SetTo(1.0F, 0.0F);
	if (n % 2 != 0)
		p[0] = -cOne;
	else
		p[0] = cOne;
	for (m = 0; m < n; m++)
	{
		if (m == 0)
		{
			for (i = 0; i < n; i++)
				for (j = 0; j < n; j++)
					c[i][j] = a[i][j];
		}
		else
		{
			for (i = 0; i < n; i++)
			{
				for (j = 0; j < n; j++)
				{
					s = cZero;
					for (k = 0; k < n; k++)
						s = s + b[i][k] * a[k][j];
					c[i][j] = s;
				}
			}
		}
		t0 = 0;
		for (i = 0; i < n; i++)		/* t0 = Trace(c) */
			t0 = t0 + c[i][i];
		t0 /= (float)(m + 1);
		p[m + 1] = -t0 * p[0];
		if (m < n - 1)
			for (i = 0; i < n; i++)
				for (j = 0; j < n; j++)
					if (j == i)
						b[i][j] = c[i][j] - t0;
					else
						b[i][j] = c[i][j];
	}
	if (n % 2 != 0)
	{
		for (i = 0; i <= n; i++)	/* p = -p */
			p[i] = -p[i];
	}
} /* CharPoly */

/* end of file CharPoly.c ----------------------------------------------- */

